;*********************************************************************
; Liest Spannung 0,00V...29,9V mit Auto-Range
; Out:			Wert in Anzeigeregister  DIGIT_1...4
; Scratch-Reg:	TEMP_A, TEMP_B
;*********************************************************************
get_spannung:
		rcall	get_adc_64			; ADC lesen, 64 Werte
;
		sbis	BER_DDR,BER_30V		; +30V
		rjmp	get_spannung_10v	; nein, 10V testen
;
		load_p	Z,99				; Test, ob auf 10V geschaltet werden mu
		sub		ZL,TEMP_A
		sbc		ZH,TEMP_B			; ja im 30V Bereich, kleiner als 9,9V?
		brcs	get_spannung_ok		; nein, ok
;
		rcall	set_10v				; kleiner, umschalten und neu messen
		rjmp	get_spannung_neu
;
get_spannung_10v:
		load_p	Z,1000				; im 10V Bereich, grer als 10V?
		sub		ZL,TEMP_A
		sbc		ZH,TEMP_B
		brcc	get_spannung_ok
;
		rcall	set_30v				; grer, umschalten und neu messen
;
get_spannung_neu:
		rcall	get_adc_64
;
get_spannung_ok:
		sbis	BER_DDR,BER_30V			; 30V an?
		rjmp	get_spannung_range_ok	; nein
;
		load_p	X,295					; 29,5V
		sub		XL,TEMP_A
		sbc		XH,TEMP_B
		brcc	get_spannung_range_ok
;
		rcall	Error					; 'E' = Error ausgeben
		rjmp	get_spannung_end
;
get_spannung_range_ok:
		rcall	show_16bit_wert
;
get_spannung_end:
		ret
;
;*********************************************************************
; Liest Logikpegel (benutzt ICP-Interrupt an PB0)
; Out:			XH, XL
; Scratch-Reg:	-
;*********************************************************************
get_logik:
		sbic	LOGIK_PORT,LOGIK_PIN			; L? 1M ist an GND
		rjmp	get_logik_signal_ok				; H, Signal liegt an
;
		sbi		BER_PORT,BER_R_IN				; 1M an +Ub
		rcall	wait_5ms
		sbis	LOGIK_PORT,LOGIK_PIN			; H?
		rjmp	get_logik_signal_ok				; L, Signal liegt an
;
		ldi		TEMP_A,0b01011111				; Eingang offen: '-'
		rjmp	get_logik_end					; ok
;
get_logik_signal_ok:
		ldi		TEMP_A,(1<<ICF1)				; wird bentigt fr Pulse < 500Hz!
		sts		TIFR1,TEMP_A					; IRQ vorsichtshalber Flag lschen
;
		rcall	wait_5ms
		lds		TEMP_A,TIFR1
		sbrc	TEMP_A,ICF1						; Pulse?
		rjmp	get_logik_pulse					; ja
;
; falls zu hohe Frequenz mal mit dem Counter probieren...
		ldi		TEMP_A,(1<<CS11)|(1<<CS12)		; Clock extern, fallende Flanke
		sts		TCCR1B,TEMP_A					; Counter starten, extern
		rcall	wait_5ms
;
		lds		TEMP_A,TCNT1L					; Counterwert holen
		lds		TEMP_B,TCNT1H
;
		sts		TCCR1B,NULL						; Counter Stop
        sts     TCNT1H,NULL						; Counter auf 0
        sts     TCNT1L,NULL
;
		tst		TEMP_B							; Pulse?
		brne	get_logik_pulse					; ja
		cpi		TEMP_A,2						; Umschaltpulse Widerstnde ausblenden
		brlo	get_logik_no_pulse				; nein
;
get_logik_pulse:
		ldi		TEMP_A,0b00001100				; Segmentmuster: 'P'
		rjmp	get_logik_end
;
get_logik_no_pulse:
		sbi		BER_DDR,BER_RC					; 10V-Bereich ein
		rcall	wait_5ms
		mov		TEMP_A,ADC_WERT_L
		mov		TEMP_B,ADC_WERT_H
;
		lds		XL,L_WERT
		lds		XH,L_WERT+1
		sub		XL,TEMP_A
		sbc		XH,TEMP_B
		brcc	get_logik_l
;
get_logik_l_h:
		lds		XL,H_WERT
		lds		XH,H_WERT+1
		sub		XL,TEMP_A
		sbc		XH,TEMP_B
		brcs	get_logik_h
;
		ldi		TEMP_A,0b11010110				; verbotener Bereich '='
		rjmp	get_logik_end
;
get_logik_l:
		ldi		TEMP_A,0b10000111				; Low: 'L'
		rjmp	get_logik_end
;
get_logik_h:
		ldi		TEMP_A,0b00001001				; High: 'H'
;
get_logik_end:
		cbi		BER_DDR,BER_RC					; 10V wieder hochohmig
		cbi		BER_PORT,BER_R_IN				; 1M wieder an GND
		rcall	wait_5ms
		sts		DIGIT_4,TEMP_A					; Symbol von oben auf Digit schreiben
		ret
;
;*********************************************************************
; Schreibt CL = Clock auf Display
; Out:			XH, XL
; Scratch-Reg:	-
;*********************************************************************
get_clock:
		ldi		TEMP_A,0b10000110				; Segmentmuster 'C'
		sts		DIGIT_4,TEMP_A
		ldi		TEMP_A,0b10000111				; Segmentmuster 'L'
		sts		DIGIT_3,TEMP_A
		ret
;
;*********************************************************************
; Misst Frequenz 0,001kHz bis ca 9MHz mit Autorange (Torzeit-basiert)
; Out:			Wert in Anzeigeregister DIGIT_1...4  
; Scratch-Reg:	TEMP_A, TEMP_B, TEMP_C, XH, XL
;
; 4 Messbereiche: Beginnend mit dem hchsten Bereich (kleinste Torzeit)
; werden die Impulse an T1 gezhlt (16 Bit). Wenn das Ergebnis nicht
; 4 stellig ist (<999), wird in den nchst kleineren Bereich geschaltet
; Torzeit*10. Bei 4 Digit (max. 9999) werden T1 Overflows nicht betrachtet.
; Nur jeweils 1 Messung: Start - X ms zhlen - Stopp, Start ...
;*********************************************************************
get_frequenz:
		load_p	X,1								; 1ms Torzeit, max Frequenz 9999kHz
		clr		TEMP_C							; TEMP_C = Digit mit Dezimalpunkten
		sbr		FLAGS_IRQ,(1<<FLAG_FREQ)		; Messung im nchsten IRQ starten
;
		rcall	check_Range						; Messung beenden, Bereich prfen
		brcs	get_frequenz_ok					; Wert > 3 Digits => fertig
;
		load_p	X,10							; sonst kleineren Bereich => 10ms
		ldi		TEMP_C,(1<<DIG_2)				; 999,9 kHz, Dezimalpunkt = Digit2
		sbr		FLAGS_IRQ,(1<<FLAG_FREQ)		; Messung im nchsten IRQ starten
;
		rcall	check_Range						; Messung beenden, Bereich prfen
		brcs	get_frequenz_ok					; Wert > 3 Digits => fertig
;
		load_p	X,100							; sonst kleineren Bereich => 100ms
		ldi		TEMP_C,(1<<DIG_3)				; 99,99 kHz, Dezimalpunkt = Digit3
		sbr		FLAGS_IRQ,(1<<FLAG_FREQ)		; Messung im nchsten IRQ starten
;
		rcall	check_Range						; Messung beenden, Bereich prfen
		brcs	get_frequenz_ok					; Wert > 3 Digits => fertig
;
		load_p	X,1000							; sonst kleineren Bereich => 1s
		ldi		TEMP_C,(1<<DIG_4)				; 9,999 kHz, Dezimalpunkt = Digit4
		sbr		FLAGS_IRQ,(1<<FLAG_FREQ)		; Messung im nchsten IRQ starten
;
get_frequenz_loop:								; kleinster Bereich
		sbrc	FLAGS_IRQ,FLAG_FREQ				; Messung Ende abwarten
		rjmp	get_frequenz_loop
;
get_frequenz_ok:
		mov		TEMP_A,CNT_WERT_L
		mov		TEMP_B,CNT_WERT_H
;
		load_p	X,9999							; berlauf?
		sub		XL,TEMP_A
		sbc		XH,TEMP_B
		brcs	get_frequenz_error
;
		mov		DIG_DP,TEMP_C					; Dezimalpunkt bergeben
		rcall	show_16bit_wert
		rjmp	get_frequenz_end
;
get_frequenz_error:
		rcall	Error							; 'E' = Error ausgeben
;
get_frequenz_end:
		sts		TCCR1B,NULL						; Counter stop
		cbr		FLAGS,(1<<FLAG_IRQ)				; Flag lschen
		ret
;
check_Range:
		sbrc	FLAGS_IRQ,FLAG_FREQ				; Messung Ende abwarten
		rjmp	check_Range
;
		load_p	X,999							; richtiger Messbereich >3 Digits?
		sub		XL,CNT_WERT_L
		sbc		XH,CNT_WERT_H					; Carry gesetzt => Bereich richtig
		ret
;
Error:	ldi		TEMP_A,0b00000110				; Segmentmuster: 'E' = Error
		sts		DIGIT_1,TEMP_A
		sts		DIGIT_2,FULL					; restliche Digits aus
		sts		DIGIT_3,FULL
		sts		DIGIT_4,FULL
		ret
