;************** IRQ - Routinen ********************************************				 

;**************************************************************************
; 1ms Timer2 Interrupt => Zeitbasis fr die Messungen & Display-Multiplex
; Scratch-Reg: -
;**************************************************************************
irq_1ms:in		SREG_SAVE,SREG
		push	TEMP_A
		push	TEMP_B
		push	ZH
		push	ZL
;
		sbrs	FLAGS,FLAG_C				; Clock Out?
		rjmp	irq_1ms_no_clock			; nein
;
		sei									; IRQ wegen Overflow Timer 0 freigeben
		rjmp	irq_1ms_no_frequenz
;
irq_1ms_no_clock:
		sbrc	FLAGS_IRQ,FLAG_FREQ			; Frequenzmessung?
		rjmp	irq_1ms_frequenz			; ja
		rjmp	irq_1ms_no_frequenz			; nein
;
irq_1ms_frequenz:
		ldi		TEMP_A,(1<<CS11)|(1<<CS12)	; Clock extern, fallende Flanke
		sts		TCCR1B,TEMP_A				; Counter starten, extern
		sbiw	X,1							; Anzahl Durchlufe
		brcc	irq_1ms_no_frequenz			; nicht fertig
;
		lds		CNT_WERT_L,TCNT1L			; Counterwert holen
		lds		CNT_WERT_H,TCNT1H
;
		cbr		FLAGS_IRQ,(1<<FLAG_FREQ)	; Flag lschen, fertig
;
		sts		TCCR1B,NULL
        sts     TCNT1H,NULL					; Counter auf 0
        sts     TCNT1L,NULL
;
irq_1ms_no_frequenz:
		lds		ADC_WERT_L,ADCL				; immer ADC-Wert holen
		lds		ADC_WERT_H,ADCH
;
; ---> Tastenabfrage
		sbrc	FLAGS_IRQ,FLAG_KEY			; letzter Tastendruck schon bearbeitet?
		rjmp	irq_1ms_key_end				; nein
;
		in		TEMP_A,KEY_PORT				; Key-Status holen
		andi	TEMP_A,(1<<KEY_PIN)			; ausmaskieren
		cp		TEMP_A,KEY_TEMP				; gleicher Zustand?
		breq	irq_1ms_key_inc				; ja
;
		mov		KEY_TEMP,TEMP_A				; nein. merken
		clr		KEY_CNT						; Zhler auf 0
		rjmp	irq_1ms_key_end
;
irq_1ms_key_inc:
		inc		KEY_CNT						; Zhler erhhen
		cpi		KEY_CNT,KEY_WAIT			; Anzahl IRQs erreicht?
		brlo	irq_1ms_key_end				; nein
;
		clr		KEY_CNT						; ja, Zhler lschen
		sbrs	KEY_TEMP,KEY_PIN			; Taste ist losgelassen
		rjmp	irq_1ms_key_press			; nein. gedrckt
;
		cbr		FLAGS_IRQ,(1<<FLAG_KEY_PRESS)	; gedrckt-Flag lschen
		rjmp	irq_1ms_key_end
;
irq_1ms_key_press:
		sbrs	FLAGS_IRQ,FLAG_KEY_PRESS	; war noch nicht losgelassen
		sbr		FLAGS_IRQ,(1<<FLAG_KEY)|(1<<FLAG_KEY_PRESS)		; Taste gltig
;
irq_1ms_key_end:
		sbr		FLAGS,(1<<FLAG_IRQ)			; Flag fr neuen Wert
;
; ---> Display-Multiplex
irq_1ms_display:
		in		TEMP_A,DIG_PORT				; Digit-Zustand holen
		andi	TEMP_A,~((1<<DIG_1)|(1<<DIG_2)|(1<<DIG_3)|(1<<DIG_4))
		ori		TEMP_A,0b00000010;			; Dezimalpunkt an PB1
		out		DIG_PORT,TEMP_A				; alle Digits & DP ausschalten
;
		ldi		ZH,high(DIGIT_1)			; erstes Digit -> H-Byte
		mov		ZL,DIG_ADR					; Digit-Nummer -> L-Byte (Offset)
		ld		TEMP_B,Z					; Segment-Muster holen
		out		SEG_PORT,TEMP_B				; und setzen
;
		or		TEMP_A, DIG_NR				; neue Digit-Nummer einschalten
		out		DIG_PORT,TEMP_A				; und setzen
;
		mov		TEMP_A, DIG_NR				; Dezimalpunkt fr aktuelles Digit?
		and		TEMP_A, DIG_DP				; steht eine 1 an gleicher Stelle?
		breq	irq_1ms_no_dp				; Sprung wenn nicht
;
		cbi		DIG_PORT,SEG_DP				; sonst Dezimalpunkt einschalten
;
irq_1ms_no_dp:
		inc		DIG_ADR						; nchste Adresse
		lsl		DIG_NR						; nchste Stelle, 0 wenn Ende
		brne	irq_1ms_end
;
		ldi		DIG_ADR,low(DIGIT_1)		; wieder auf Anfang
		ldi		DIG_NR,(1<<DIG_1)
;
irq_1ms_end:
		pop		ZL
		pop		ZH
		pop		TEMP_B
		pop		TEMP_A
		out		SREG,SREG_SAVE
		reti
;
;*********************************************************************
; Overflow Timer0 Interrupt, erzeugt konstante Frequenz an PC1
; Scratch-Reg: -
;*********************************************************************
irq_overflow:								; bei 24,576MHz Quarz
		in		SREG_SAVE,SREG				; und Prescaler = 8
		push	TEMP_A						; ca 100kHz Output
		push	TEMP_B
;
		ldi     TEMP_A,241					; 256-(24,576MHz * 10us/2) / 8 = 240,64
		out     TCNT0,TEMP_A				; Timer-Wert wieder setzen
;
		ldi		TEMP_B,(1<<CLK_OUT)
		in		TEMP_A,BER_PORT				; Clock-Pin PC1 einlesen
		eor		TEMP_A,TEMP_B				; Bit toggeln
		out		BER_PORT,TEMP_A				; und wieder ausgeben
;
		pop		TEMP_B
		pop		TEMP_A
		out		SREG,SREG_SAVE
		reti
