; ATMega8 Mess-Stick / Multimeter
; 10.05.2007 M'Soft Berlin (VER = 0, REV = 5)
;
; 19.09.2015 Thomas N, wesentliche nderungen
;	- Umstellung von ATmega8 @16MHz auf ATmega48 @24,576MHz (Register, Timing, ADC..)
;	- Statt Periodendauer-Messung ebenfalls Frequenzmessung, dann 1Hz - 9999Hz
;	  ohne 1:4 Prescaler 7473, max Frequenz dank hohem Prozessortakt 9999kHz => E
;	- Bei Frequenzmessung Anzeige der Messfolge durch blinkenden Dezimalpunkt rechts
;	- Anpassungen fr LED-Display mit gemeinsamer Anode ohne Treibertransistoren
;	- seg_tab in subroutinen.inc integriert nur noch fr 0-9 (tabellen.inc entfllt)
;	  Sonderzeichen "P", "C", "L" etc. werden direkt zugewiesen
;	- Secure-Bits fr Mega48: gesetzt SUT0, BODLEVEL1&2, ggf. SPIEN
;
.include "m48def.inc"
.include "Definitionen.inc"					; Ports, Register, Flags etc.
;
;****************************** Vektor-Liste *************************
.CSEG
					rjmp    Reset
.ORG	OC2Aaddr	rjmp	irq_1ms			; Timer/Counter2 Compare Match
.ORG	OVF0addr	rjmp	irq_overflow	; Timer/Counter0 Overflow
;
;*********************************************************************
; Initialisierungsroutine
;*********************************************************************
.ORG	INT_VECTORS_SIZE
Reset:	ldi     TEMP_A,low(RAMEND)			; Hardware initialisieren
		out     SPL,TEMP_A					; Stack an das interne RAM-Ende
		ldi     TEMP_A,high(RAMEND)
		out     SPH,TEMP_A
		clr		NULL						; wird sogleich bentigt!
;
; Port B *************************************************************
; PB0/ICP = Input Logik-Pulsmessung, PB1=DP (Segment), PB2-PB5 = DIG_1-4
		ldi		TEMP_A, 0b00000010
		out     PORTB,TEMP_A				; alle Digits aus, DP aus
		ldi     TEMP_A, 0b00111110;
		out     DDRB,TEMP_A					; 1=Ausgang, 0=Eingang
;
; Port C *************************************************************
; PC1=Clock out, PC2=Mode-Taste, PC4,5= DVM-Range-Widerstnde, PC0 = ADC/DVM, PC3=ADC/Logik
		ldi		TEMP_A,(1<<KEY_PIN)			; PullUp fr Taste ein,			
		out     PORTC,TEMP_A				; RC an GND, R_IN an GND
		ldi		TEMP_A,(1<<CLK_OUT)|(1<<BER_RC)|(1<<BER_R_IN)
		out     DDRC,TEMP_A
;
; Port D *************************************************************
; Segment-Treiber: PD0-PD4 = A-E, PD6,7 = F,G, PD5=T1=Eingang Frequenzmessung
		ldi     TEMP_A, 0b11011111
		out     PORTD,TEMP_A               	; Alle Segmente aus
		out     DDRD,TEMP_A					; Segmente = Ausgang
;
; TIMER 2 - 1ms (8Bit) ***********************************************
; Vorteiler 128, Timer 191 => liefert die Zeitbasis fr Frequenzmessung
		ldi		TEMP_A,(1<<WGM21)			; CTC 
		sts		TCCR2A,TEMP_A
		ldi		TEMP_A,(1<<CS22)|(1<<CS20)	; Prescaler 128-fach
		sts		TCCR2B,TEMP_A
		ldi     TEMP_A,TCONST_TIME			; Timer-Wert (191)
		sts     OCR2A,TEMP_A
		sts		ASSR,NULL
		ldi		TEMP_A,(1<<OCIE2A)			; Timer-IRQ frei
		sts     TIMSK2,TEMP_A
;
; TIMER 1 - Counter (16Bit) ******************************************
; T1 = Frequenzzhler-Eingang ohne Overflow-Interrupt!
		sts		TCCR1A,NULL					; Counter stop
		sts		TCCR1B,NULL
		sts     TCNT1H,NULL			; Counter auf 0	-> Grundstellung in allen
		sts     TCNT1L,NULL			; Funktionen am Ende wiederherstellen!
;
; TIMER 0 - extern Clock (8Bit) **************************************
; erzeugt konstante Frequenz auf Ausgang PC1
		ldi		TEMP_A,(1<<CS01)			; Prescaler 8-fach
		out		TCCR0B,TEMP_A				; Preload in IRQ.INC
;
; ADC ****************************************************************
		ldi		TEMP_A,(1<<REFS0)|(1<<REFS1); interne Referenz
		sts		ADMUX,TEMP_A
		ldi		TEMP_A,(1<<ADEN)|(1<<ADSC)|(1<<ADATE)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0)
		sts		ADCSRA,TEMP_A				; Free Running, Prescaler = 128
		sts		ADCSRB,NULL					; ergibt 192kHz ADC-Takt @24,576MHz
;
;*********************************************************************
; Software initialisieren
		ldi		FLAGS,(1<<FLAG_U)			; Start mit Funktion Spannungsmesser
		clr		FLAGS_IRQ
		clr		DIG_DP						; alle Dezimalpunkte aus
		ldi		DIG_NR,(1<<DIG_1)			; Bit fr aktuelle Digitposition Digit 1
		ldi		DIG_ADR,low(DIGIT_1)		; Adresse Daten fr Digit 1 (einer)
		ldi		TEMP_A,0b11111111
		mov		FULL, TEMP_A				; Zum Lschen der Segmente
;
		sts		DIGIT_1,FULL				; Segmente aller Digits aus
		sts		DIGIT_2,FULL
		sts		DIGIT_3,FULL
		sts		DIGIT_4,FULL
;
		sei
;
		rcall	init						; Mit Ub und setzt Spannungs- und Logikparameter
;
;*********************************************************************
; Main-Loop
;*********************************************************************
main:	sbrs	FLAGS,FLAG_U			; Spannung messen?
		rjmp	main_freq				; nein
		rcall	set_10v					; ja => Start im unteren Mebereich
;
main_span:
		rcall	get_spannung
		ldi		TEMP_D,100				; Warteschleife, 100ms -> 10 Messungen/s
		rjmp	main_wait_loop
;
main_freq:
		sbrs	FLAGS,FLAG_F			; Frequenz messen?
		rjmp	main_logik				; nein
		rcall	get_frequenz
;
		com		BLINK
		sbrc	BLINK,0					; Messfolge-Blinker
		ori		DIG_DP, 0b00000100		; Dezimalpunkt an Digit1=PB2 toggeln
;
		ldi		TEMP_D,100				; Warteschleife, 100ms -> max 10 Messungen/s
		rjmp	main_wait_loop
;
main_logik:
		sbrs	FLAGS,FLAG_L			; Logikpegel?
		rjmp	main_clock				; nein
		rcall	get_logik
		sbrs	FLAGS_IRQ,FLAG_KEY		; Taste gedrckt?
		rjmp	main_logik				; nein
		rjmp	main_wait_key
;
main_clock:
		rcall	get_clock
		sbrs	FLAGS_IRQ,FLAG_KEY		; Taste gedrckt?
		rjmp	main_clock				; nein
		rjmp	main_wait_key
;
main_wait_loop:
		sbrs	FLAGS,FLAG_IRQ			; auf IRQ warten
		rjmp	main_wait_loop
;
		cbr		FLAGS,(1<<FLAG_IRQ)		; Flag lschen fr nchsten IRQ
;
		sbrs	FLAGS_IRQ,FLAG_KEY		; Taste gedrckt?
		rjmp	main_wait_no_key		; nein
;
main_wait_key:
		rcall	key_decode				; Taste auswerten, wechselt Funktion U->F->L->C->U usw.
		rjmp	main_key_end
;
main_wait_no_key:
		dec		TEMP_D					; Warteschleife bei U und F
		brne	main_wait_loop
;
main_key_end:
		sbrc	FLAGS,FLAG_U			; Spannung messen
		rjmp	main_span
;
		sbrc	FLAGS,FLAG_F			; Frequenz messen
		rjmp	main_freq
;
		sbrc	FLAGS,FLAG_L			; Logik messen
		rjmp	main_logik
;
		sbrc	FLAGS,FLAG_C			; Clock ausgeben
		rjmp	main_clock
;
		rjmp	main					; hier sollten wir nie ankommen... ;)
;
;*********************************************************************				 
;
.include "Funktionen.inc"
.include "Subroutinen.inc"
.include "Irq.inc"
